#!/usr/bin/env python2
# -*- coding: utf-8 -*-
##################################################
# GNU Radio Python Flow Graph
# Title: interferometer simulator (IOR1)
# Author: Graham Woan
# Description: a simple total-power radio telescope
# Generated: Wed Nov  7 21:38:01 2018
##################################################

if __name__ == '__main__':
    import ctypes
    import sys
    if sys.platform.startswith('linux'):
        try:
            x11 = ctypes.cdll.LoadLibrary('libX11.so')
            x11.XInitThreads()
        except:
            print "Warning: failed to XInitThreads()"

from PyQt4 import Qt
from gnuradio import analog
from gnuradio import blocks
from gnuradio import eng_notation
from gnuradio import filter
from gnuradio import gr
from gnuradio import qtgui
from gnuradio.eng_option import eng_option
from gnuradio.filter import firdes
from gnuradio.qtgui import Range, RangeWidget
from optparse import OptionParser
import math,cmath
import sip
import sys
from gnuradio import qtgui


class interferometer_simulator(gr.top_block, Qt.QWidget):

    def __init__(self):
        gr.top_block.__init__(self, "interferometer simulator (IOR1)")
        Qt.QWidget.__init__(self)
        self.setWindowTitle("interferometer simulator (IOR1)")
        qtgui.util.check_set_qss()
        try:
            self.setWindowIcon(Qt.QIcon.fromTheme('gnuradio-grc'))
        except:
            pass
        self.top_scroll_layout = Qt.QVBoxLayout()
        self.setLayout(self.top_scroll_layout)
        self.top_scroll = Qt.QScrollArea()
        self.top_scroll.setFrameStyle(Qt.QFrame.NoFrame)
        self.top_scroll_layout.addWidget(self.top_scroll)
        self.top_scroll.setWidgetResizable(True)
        self.top_widget = Qt.QWidget()
        self.top_scroll.setWidget(self.top_widget)
        self.top_layout = Qt.QVBoxLayout(self.top_widget)
        self.top_grid_layout = Qt.QGridLayout()
        self.top_layout.addLayout(self.top_grid_layout)

        self.settings = Qt.QSettings("GNU Radio", "interferometer_simulator")
        self.restoreGeometry(self.settings.value("geometry").toByteArray())

        ##################################################
        # Variables
        ##################################################
        self.samp_rate = samp_rate = 1000000
        self.int_time = int_time = 10.0/samp_rate*50
        self.f_0 = f_0 = 100e6
        self.delay = delay = 0.0000
        self.dec = dec = int(samp_rate/10.0*int_time)
        self.T_s = T_s = 1
        self.T_a = T_a = 2

        ##################################################
        # Blocks
        ##################################################
        self._int_time_range = Range(10.0/samp_rate, 10.0/samp_rate*100, 10.0/samp_rate, 10.0/samp_rate*50, 200)
        self._int_time_win = RangeWidget(self._int_time_range, self.set_int_time, 'integration time', "dial", float)
        self.top_grid_layout.addWidget(self._int_time_win, 2,1)
        self._delay_range = Range(-1.0/f_0/2.0, 1.0/f_0/2.0, 1.0/f_0/36.0/2.0, 0.0000, 200)
        self._delay_win = RangeWidget(self._delay_range, self.set_delay, 'delay', "dial", float)
        self.top_grid_layout.addWidget(self._delay_win, 3,1)
        self._T_s_range = Range(0, 5, 0.05, 1, 100)
        self._T_s_win = RangeWidget(self._T_s_range, self.set_T_s, 'system temp', "dial", float)
        self.top_grid_layout.addWidget(self._T_s_win, 0,1)
        self._T_a_range = Range(0, 4, 0.05, 2, 100)
        self._T_a_win = RangeWidget(self._T_a_range, self.set_T_a, 'antenna temp', "dial", float)
        self.top_grid_layout.addWidget(self._T_a_win, 1,1)
        self.rational_resampler_xxx_2_0 = filter.rational_resampler_ccc(
                interpolation=1,
                decimation=dec,
                taps=None,
                fractional_bw=None,
        )
        self.rational_resampler_xxx_2 = filter.rational_resampler_ccc(
                interpolation=1,
                decimation=10,
                taps=None,
                fractional_bw=None,
        )
        self.rational_resampler_xxx_1_0 = filter.rational_resampler_ccc(
                interpolation=1,
                decimation=10,
                taps=None,
                fractional_bw=None,
        )
        self.rational_resampler_xxx_1 = filter.rational_resampler_ccc(
                interpolation=1,
                decimation=dec,
                taps=None,
                fractional_bw=None,
        )
        self.qtgui_const_sink_x_1_0 = qtgui.const_sink_c(
        	150, #size
        	'radio interferometer signals in the complex plane', #name
        	3 #number of inputs
        )
        self.qtgui_const_sink_x_1_0.set_update_time(0.01)
        self.qtgui_const_sink_x_1_0.set_y_axis(-1, 1)
        self.qtgui_const_sink_x_1_0.set_x_axis(-1, 1)
        self.qtgui_const_sink_x_1_0.set_trigger_mode(qtgui.TRIG_MODE_FREE, qtgui.TRIG_SLOPE_POS, 0.0, 0, "")
        self.qtgui_const_sink_x_1_0.enable_autoscale(False)
        self.qtgui_const_sink_x_1_0.enable_grid(True)
        self.qtgui_const_sink_x_1_0.enable_axis_labels(False)

        if not True:
          self.qtgui_const_sink_x_1_0.disable_legend()

        labels = ['antenna 1', 'antenna 2', 'cross correlation', '', '',
                  '', '', '', '', '']
        widths = [1, 1, 1, 1, 1,
                  1, 1, 1, 1, 1]
        colors = ["blue", "green", "red", "red", "red",
                  "red", "red", "red", "red", "red"]
        styles = [0, 0, 0, 0, 0,
                  0, 0, 0, 0, 0]
        markers = [0, 0, 0, 0, 0,
                   0, 0, 0, 0, 0]
        alphas = [1, 1, 0.5, 1.0, 1.0,
                  1.0, 1.0, 1.0, 1.0, 1.0]
        for i in xrange(3):
            if len(labels[i]) == 0:
                self.qtgui_const_sink_x_1_0.set_line_label(i, "Data {0}".format(i))
            else:
                self.qtgui_const_sink_x_1_0.set_line_label(i, labels[i])
            self.qtgui_const_sink_x_1_0.set_line_width(i, widths[i])
            self.qtgui_const_sink_x_1_0.set_line_color(i, colors[i])
            self.qtgui_const_sink_x_1_0.set_line_style(i, styles[i])
            self.qtgui_const_sink_x_1_0.set_line_marker(i, markers[i])
            self.qtgui_const_sink_x_1_0.set_line_alpha(i, alphas[i])

        self._qtgui_const_sink_x_1_0_win = sip.wrapinstance(self.qtgui_const_sink_x_1_0.pyqwidget(), Qt.QWidget)
        self.top_grid_layout.addWidget(self._qtgui_const_sink_x_1_0_win, 0,0,4,1)
        self.low_pass_filter_0_0 = filter.fir_filter_ccf(10, firdes.low_pass(
        	0.1, samp_rate, samp_rate/2.0/10.0, samp_rate/2.0/100.0, firdes.WIN_RECTANGULAR, 6.76))
        self.low_pass_filter_0 = filter.fir_filter_ccf(dec, firdes.low_pass(
        	1, int(samp_rate/10.0), 1.0/int_time, 1.0/int_time/2.0, firdes.WIN_RECTANGULAR, 6.76))
        self.blocks_throttle_0_1 = blocks.throttle(gr.sizeof_gr_complex*1, samp_rate,True)
        self.blocks_throttle_0_0 = blocks.throttle(gr.sizeof_gr_complex*1, samp_rate,True)
        self.blocks_throttle_0 = blocks.throttle(gr.sizeof_gr_complex*1, samp_rate,True)
        self.blocks_multiply_xx_0 = blocks.multiply_vcc(1)
        self.blocks_multiply_const_vxx_0 = blocks.multiply_const_vcc((cmath.exp(-2*math.pi*1j*f_0*delay), ))
        (self.blocks_multiply_const_vxx_0).set_block_alias("phase rotate")
        self.blocks_conjugate_cc_0 = blocks.conjugate_cc()
        self.blocks_add_xx_0_0 = blocks.add_vcc(1)
        self.blocks_add_xx_0 = blocks.add_vcc(1)
        self.analog_noise_source_x_0_1 = analog.noise_source_c(analog.GR_GAUSSIAN, T_s, -576948462)
        self.analog_noise_source_x_0_0 = analog.noise_source_c(analog.GR_GAUSSIAN, T_s, -324323)
        self.analog_noise_source_x_0 = analog.noise_source_c(analog.GR_GAUSSIAN, T_a, -344843)
        (self.analog_noise_source_x_0).set_block_alias("signal")

        ##################################################
        # Connections
        ##################################################
        self.connect((self.analog_noise_source_x_0, 0), (self.blocks_throttle_0, 0))
        self.connect((self.analog_noise_source_x_0_0, 0), (self.blocks_throttle_0_0, 0))
        self.connect((self.analog_noise_source_x_0_1, 0), (self.blocks_throttle_0_1, 0))
        self.connect((self.blocks_add_xx_0, 0), (self.blocks_conjugate_cc_0, 0))
        self.connect((self.blocks_add_xx_0, 0), (self.rational_resampler_xxx_1_0, 0))
        self.connect((self.blocks_add_xx_0_0, 0), (self.blocks_multiply_xx_0, 1))
        self.connect((self.blocks_add_xx_0_0, 0), (self.rational_resampler_xxx_2, 0))
        self.connect((self.blocks_conjugate_cc_0, 0), (self.blocks_multiply_xx_0, 0))
        self.connect((self.blocks_multiply_const_vxx_0, 0), (self.blocks_add_xx_0_0, 0))
        self.connect((self.blocks_multiply_xx_0, 0), (self.low_pass_filter_0_0, 0))
        self.connect((self.blocks_throttle_0, 0), (self.blocks_add_xx_0, 1))
        self.connect((self.blocks_throttle_0, 0), (self.blocks_multiply_const_vxx_0, 0))
        self.connect((self.blocks_throttle_0_0, 0), (self.blocks_add_xx_0_0, 1))
        self.connect((self.blocks_throttle_0_1, 0), (self.blocks_add_xx_0, 0))
        self.connect((self.low_pass_filter_0, 0), (self.qtgui_const_sink_x_1_0, 2))
        self.connect((self.low_pass_filter_0_0, 0), (self.low_pass_filter_0, 0))
        self.connect((self.rational_resampler_xxx_1, 0), (self.qtgui_const_sink_x_1_0, 1))
        self.connect((self.rational_resampler_xxx_1_0, 0), (self.rational_resampler_xxx_2_0, 0))
        self.connect((self.rational_resampler_xxx_2, 0), (self.rational_resampler_xxx_1, 0))
        self.connect((self.rational_resampler_xxx_2_0, 0), (self.qtgui_const_sink_x_1_0, 0))

    def closeEvent(self, event):
        self.settings = Qt.QSettings("GNU Radio", "interferometer_simulator")
        self.settings.setValue("geometry", self.saveGeometry())
        event.accept()

    def get_samp_rate(self):
        return self.samp_rate

    def set_samp_rate(self, samp_rate):
        self.samp_rate = samp_rate
        self.set_int_time(10.0/self.samp_rate*50)
        self.set_dec(int(self.samp_rate/10.0*self.int_time))
        self.low_pass_filter_0_0.set_taps(firdes.low_pass(0.1, self.samp_rate, self.samp_rate/2.0/10.0, self.samp_rate/2.0/100.0, firdes.WIN_RECTANGULAR, 6.76))
        self.low_pass_filter_0.set_taps(firdes.low_pass(1, int(self.samp_rate/10.0), 1.0/self.int_time, 1.0/self.int_time/2.0, firdes.WIN_RECTANGULAR, 6.76))
        self.blocks_throttle_0_1.set_sample_rate(self.samp_rate)
        self.blocks_throttle_0_0.set_sample_rate(self.samp_rate)
        self.blocks_throttle_0.set_sample_rate(self.samp_rate)

    def get_int_time(self):
        return self.int_time

    def set_int_time(self, int_time):
        self.int_time = int_time
        self.set_dec(int(self.samp_rate/10.0*self.int_time))
        self.low_pass_filter_0.set_taps(firdes.low_pass(1, int(self.samp_rate/10.0), 1.0/self.int_time, 1.0/self.int_time/2.0, firdes.WIN_RECTANGULAR, 6.76))

    def get_f_0(self):
        return self.f_0

    def set_f_0(self, f_0):
        self.f_0 = f_0
        self.blocks_multiply_const_vxx_0.set_k((cmath.exp(-2*math.pi*1j*self.f_0*self.delay), ))

    def get_delay(self):
        return self.delay

    def set_delay(self, delay):
        self.delay = delay
        self.blocks_multiply_const_vxx_0.set_k((cmath.exp(-2*math.pi*1j*self.f_0*self.delay), ))

    def get_dec(self):
        return self.dec

    def set_dec(self, dec):
        self.dec = dec

    def get_T_s(self):
        return self.T_s

    def set_T_s(self, T_s):
        self.T_s = T_s
        self.analog_noise_source_x_0_1.set_amplitude(self.T_s)
        self.analog_noise_source_x_0_0.set_amplitude(self.T_s)

    def get_T_a(self):
        return self.T_a

    def set_T_a(self, T_a):
        self.T_a = T_a
        self.analog_noise_source_x_0.set_amplitude(self.T_a)


def main(top_block_cls=interferometer_simulator, options=None):

    from distutils.version import StrictVersion
    if StrictVersion(Qt.qVersion()) >= StrictVersion("4.5.0"):
        style = gr.prefs().get_string('qtgui', 'style', 'raster')
        Qt.QApplication.setGraphicsSystem(style)
    qapp = Qt.QApplication(sys.argv)

    tb = top_block_cls()
    tb.start()
    tb.show()

    def quitting():
        tb.stop()
        tb.wait()
    qapp.connect(qapp, Qt.SIGNAL("aboutToQuit()"), quitting)
    qapp.exec_()


if __name__ == '__main__':
    main()
